﻿using UnityEngine;
using System.Collections;

public class Player : MonoBehaviour
{

	public Transform muzzle;
	public Projectile projectile;

	public float moveSpeed = 5;
	Camera mainCamera;
	Vector3 moveVelocity;
	Rigidbody myRigidbody;

	bool isJump = false;

	void Start ()
	{
		mainCamera = Camera.main;
		myRigidbody = GetComponent<Rigidbody> ();
	}

	void Update ()
	{
		// 移動の入力処理
		var moveInput = new Vector3 (Input.GetAxisRaw ("Horizontal"), 0, Input.GetAxisRaw ("Vertical"));
		moveVelocity = moveInput * moveSpeed;

		// プレイヤー向きの処理
		Ray ray = mainCamera.ScreenPointToRay (Input.mousePosition);
		Plane groundPlane = new Plane (Vector3.up, Vector3.zero); // 計算用XZ平面
		float rayDistance;
		// XY平面と光線が交差したところまでの、光線の距離がrayDistanceで取得。
		if (groundPlane.Raycast (ray, out rayDistance)) {
			// 光線の距離rayDistanceが分かったので、座標を取得。
			Vector3 point = ray.GetPoint (rayDistance);
			Debug.DrawLine (ray.origin, point, Color.yellow);
			Debug.DrawLine (transform.position, point, Color.black);
			// 下を向かないように、Playerの高さにする。
			Vector3 heightCorrectedPoint = new Vector3 (point.x, transform.position.y, point.z);
			transform.LookAt (heightCorrectedPoint);
			Debug.DrawLine (transform.position, heightCorrectedPoint, Color.gray);
		}

		// ショット
		if (Input.GetButtonDown ("Fire1")) {
			Shoot ();
		}

		// ジャンプ
		if (Input.GetButtonDown ("Jump")) {
			isJump = true;
		}
	}

	void FixedUpdate ()
	{
		// MovePositionは、rigidbodyのinterpolation(補間)設定に従って移動する。
		myRigidbody.MovePosition (myRigidbody.position + moveVelocity * Time.fixedDeltaTime);

		Jump ();
	}

	void Shoot ()
	{
		var bullet = Instantiate (projectile, muzzle.position, muzzle.rotation) as Projectile;
		Debug.Log ("Shot: " + bullet);
	}

	void Jump ()
	{
		if (isJump) {
			isJump = false;
			float force = 5f;
			myRigidbody.AddForce (Vector3.up * force, ForceMode.Impulse);
		}
	}

	// Debug Code
	void OnDrawGizmos ()
	{
		Gizmos.color = Color.blue;
		// ワールド座標に変換
		Vector3 direction = transform.TransformDirection (Vector3.forward) * 3; // 向き
		Gizmos.DrawRay (transform.position, direction);
	}
}
